import { LastFM } from './base';
import { LastFMApiRequest, LastFMRequestParams, LastFMUnknownFunction } from './api-request';
export interface LastFMTrackParams extends LastFMRequestParams<number | void> {
    readonly track: string;
    readonly artist: string;
}
export interface LastFMTrackOptionalParams {
    readonly mbid?: string;
    readonly autocorrect?: 0 | 1;
}
export interface LastFMTrackAddTags extends LastFMTrackParams {
    readonly tags: string | string[];
}
export interface LastFMTrackGetInfo extends LastFMTrackParams, LastFMTrackOptionalParams {
    readonly username?: string;
}
export interface LastFMTrackGetSimilar extends LastFMTrackParams, LastFMTrackOptionalParams {
    readonly limit?: number;
}
export interface LastFMTrackGetTags extends LastFMTrackParams, LastFMTrackOptionalParams {
    readonly user?: string;
}
export interface LastFMTrackGetTopTags extends LastFMTrackParams, LastFMTrackOptionalParams {
}
export interface LastFMTrackRemoveTag extends LastFMTrackParams {
    readonly tag: string;
}
export interface LastFMTrackScrobble extends LastFMTrackParams {
    readonly timestamp?: number;
    readonly mbid?: string;
    readonly album?: string;
    readonly context?: string;
    readonly streamId?: string;
    readonly duration?: number;
    readonly chosenByUser?: 0 | 1;
    readonly trackNumber?: number;
    readonly albumArtist?: string;
}
export interface LastFMTrackSearch extends LastFMRequestParams<number | void> {
    readonly page?: number;
    readonly limit?: number;
    readonly track: string;
    readonly artist?: string;
}
export interface LastFMTrackUpdateNowPlaying extends LastFMTrackParams {
    readonly mbid?: string;
    readonly album?: string;
    readonly context?: string;
    readonly duration?: number;
    readonly trackNumber?: number;
    readonly albumArtist?: string;
}
export declare class LastFMTrack extends LastFM {
    constructor(apiKey: string, secret?: string, sessionKey?: string);
    addTags(params: LastFMTrackAddTags, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getCorrection(params: LastFMTrackParams, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getInfo(params: LastFMTrackGetInfo, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getSimilar(params: LastFMTrackGetSimilar, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getTags(params: LastFMTrackGetTags, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getTopTags(params: LastFMTrackGetTopTags, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    love(params: LastFMTrackParams, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    removeTag(params: LastFMTrackRemoveTag, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    scrobble(params: LastFMTrackScrobble, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    scrobbleMany(paramsArr: LastFMTrackScrobble[], callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    search(params: LastFMTrackSearch, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    unlove(params: LastFMTrackParams, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    updateNowPlaying(params: LastFMTrackUpdateNowPlaying, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
}
