import { LastFM } from './base';
import { LastFMParam, LastFMApiRequest, LastFMRequestParams, LastFMUnknownFunction } from './api-request';
export interface LastFMUserParams extends LastFMRequestParams<LastFMParam | number | void | boolean> {
    readonly user: string;
}
export interface LastFMUserOptionalParams {
    readonly page?: number;
    readonly limit?: number;
}
export interface LastFMUserGetFriends extends LastFMUserParams, LastFMUserOptionalParams {
    readonly recenttracks?: boolean;
}
export interface LastFMUserGetLovedTracks extends LastFMUserParams, LastFMUserOptionalParams {
}
export interface LastFMUserGetPersonalTags extends LastFMUserParams, LastFMUserOptionalParams {
    readonly tag?: string;
    readonly taggingtype: 'artist' | 'album' | 'track';
}
export interface LastFMUserGetRecentTracks extends LastFMUserParams, LastFMUserOptionalParams {
    readonly to?: string;
    readonly from?: string;
    readonly extended: 0 | 1;
}
export interface LastFMUserGetTop extends LastFMUserParams, LastFMUserOptionalParams {
    readonly period?: 'overall' | '7day' | '1month' | '3month' | '6month' | '12month';
}
export interface LastFMUserGetTopTags extends LastFMRequestParams<string | number | void> {
    readonly user: string;
    readonly limit?: number;
}
export interface LastFMUserGetWeekly extends LastFMRequestParams<string | number | void> {
    readonly to?: string;
    readonly user: string;
    readonly from?: string;
}
export declare class LastFMUser extends LastFM {
    constructor(apiKey: string, secret?: string, sessionKey?: string);
    getFriends(params: LastFMUserGetFriends, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getInfo(params: LastFMUserParams, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getLovedTracks(params: LastFMUserGetLovedTracks, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getPersonalTags(params: LastFMUserGetPersonalTags, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getRecentTracks(params: LastFMUserGetRecentTracks, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getTopAlbums(params: LastFMUserGetTop, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getTopArtists(params: LastFMUserGetTop, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getTopTags(params: LastFMUserGetTopTags, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getTopTracks(params: LastFMUserGetTop, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getWeeklyAlbumChart(params: LastFMUserGetWeekly, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getWeeklyArtistChart(params: LastFMUserGetWeekly, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getWeeklyChartList(params: LastFMUserParams, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
    getWeeklyTrackChart(params: LastFMUserGetWeekly, callback: LastFMUnknownFunction): Promise<LastFMApiRequest> | void;
}
